package de.fitko.fitconnect.impl;

import com.google.inject.Inject;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.event.SecurityEventToken;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionRequest;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.internal.AuthenticationException;
import de.fitko.fitconnect.api.services.Subscriber;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.validation.MetadataValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Optional;


public class SubmissionSubscriber implements Subscriber {

    private static final Logger logger = LoggerFactory.getLogger(SubmissionSubscriber.class);

    private final OAuthService authService;
    private final CryptoService cryptoService;
    private final MetadataValidator metadataValidator;

    @Inject
    public SubmissionSubscriber(final OAuthService authService,
                                final CryptoService cryptoService,
                                final MetadataValidator metadataValidator){
        this.authService = authService;
        this.cryptoService = cryptoService;
        this.metadataValidator = metadataValidator;
    }

    @Override
    public Optional<OAuthToken> retrieveOAuthToken(String clientId, String clientSecret, String... scope) {
        try {
            return Optional.of(authService.authenticate(clientId, clientSecret, scope));
        } catch (AuthenticationException e) {
            logger.error("Retrieving the OAuth token failed: ", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public Optional<Data> decryptSubmissionData(RSAKey privateKey, Data encryptedData) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Attachment> decryptAttachment(RSAKey privateKey, Attachment encryptedAttachment) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateMetadataSchema(Metadata metadata, String jsonSchema) {
        return metadataValidator.validateMetadataSchema(metadata, jsonSchema);
    }

    @Override
    public ValidationResult validateMetadataHashValues(Metadata metadata) {
        return metadataValidator.validateMetadataHashValues(metadata);
    }

    @Override
    public List<SubmissionRequest> pollAvailableSubmissions(String destinationId, int limit, int offset) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<SubmissionRequest> getSubmission(String submissionId) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<SecurityEventToken> createSecurityEventToken(Data data, Attachment attachment, RSAKey privateKey) {
        throw new UnsupportedOperationException("not yet implemented");
    }
}
